unit UProto;

interface
uses
   Graphics, math;

type
  Mat3D  = record
    nc,nr  : integer;
    dat    : array of array of array of single;
  end;

  function aj255(z : single) : integer;

  procedure BM2Mat (B  : TBitMap; var MM   : Mat3D);
  procedure Mat2BM (MM : Mat3D  ; var BB   : TBitMap);
  procedure Mat2Mat(Ma : Mat3D  ; var Mb   : Mat3D);
  procedure Intercambio(var Ma : Mat3D  ; var Mb : Mat3D);

var
  kan  : array [0..2] of boolean;

implementation

function aj255(z : single) : integer;
begin
  if (z<0)
   then Result := 0
   else if (z>255)
     then Result := 255
     else Result := ceil(z);
end;

procedure BM2Mat(B : TBitMap; var MM : Mat3D);
var
 x,y, nx,ny, Pix : integer;
begin
  nx := MM.nc;
  ny := MM.nr;
  SetLength(MM.dat, nx,ny,3);

  for y := 0 to ny - 1 do
    for x := 0 to nx - 1 do begin
    Pix := B.Canvas.Pixels[x,y];
      MM.dat[x][y][0] :=   Pix and $FF           ; // canal rojo
      MM.dat[x][y][1] :=   (Pix  SHR 8) and $FF  ; // canal verde
      MM.dat[x][y][2] :=   (Pix SHR 16) and $FF  ; // canal azul
    end;
end;

procedure Mat2BM(MM : Mat3D  ; var BB   : TBitMap);
var
 x,y, nx,ny, Pix : integer;
 r,g,b           : integer;
begin
  nx := MM.nc;
  ny := MM.nr;
  BB.Width  := nx;
  BB.Height := ny;

  for y := 0 to ny - 1 do
    for x := 0 to nx - 1 do begin
      r := aj255(MM.dat[x][y][0]) ; // canal rojo
      g := aj255(MM.dat[x][y][1]) ; // canal verde
      b := aj255(MM.dat[x][y][2]) ; // canal azul
      Pix := r + (g SHL 8) + (b SHL 16);

      BB.Canvas.Pixels[x,y] := Pix;
    end;
end;

// Copiar Ma en Mb
procedure Mat2Mat(Ma : Mat3D; var Mb : Mat3D);
var
 x,y, nx,ny, cc: integer;
begin
  nx := Ma.nc;
  ny := Ma.nr;

  Mb.nc := nx;
  Mb.nr := ny;

  SetLength(Mb.dat, nx,ny,3);

  for y := 0 to ny - 1 do
    for x := 0 to nx - 1 do
      for cc := 0 to 2 do
        Mb.dat[x][y][cc] := Ma.dat[x][y][cc] ;
  end;

// Intercambia contenido de 2 matrices de 3 canales
procedure Intercambio(var Ma : Mat3D  ; var Mb : Mat3D);
var
 x,y, nx,ny, cc : integer;
 temp           : single;

begin
  // si las Matrices son del mismo tamao
  if (Ma.nc=Mb.nc) and (Ma.nr=Mb.nr) then begin
    nx := Ma.nc;
    ny := Ma.nr;
    for y := 0 to ny - 1 do
      for x := 0 to nx - 1 do
        for cc := 0 to 2 do begin
           temp := Mb.dat[x][y][cc] ;
           Mb.dat[x][y][cc] := Ma.dat[x][y][cc] ;
           Ma.dat[x][y][cc] := temp;
        end;
  end;
end;

// tareas de inicializacion
begin
  // activamos todos los canales
  kan[0] := true;
  kan[1] := true;
  kan[2] := true;

end.
